<?php
/**
 * @file
 * Pass HTMLTidy over some inputs to make them valid XHTML
 * In the commandline version, the xhtml_tidy.conf file
 * is read from the same directory as this library.
 */
 
@require_once('debug.inc');

if(! defined("TIDY_PATH")){
  define("TIDY_PATH",system_path("/bin/")); //Include trailing slash
}
define("TIDY_OPTIONS"," -q -config ".dirname(__FILE__)."\\xhtml_tidy.conf ");


/**
 * Use the tidy extension (Whee!) to fix randomly downloaded 'html'
 *
 * The result should be valid XHTML that can now be converted into a DOM
 * if the extension is unavailable, try doing it on the commandline
 * if the executable is available.
 *
 * @param $data string Raw web page source
 * @return $string Cooked web page source
 */
function xml_tidy_string($data)
{
  // Because HTML character entities should have been properly declared (but naturally are not) &nbsp; and friends will cause complaints.
  // Decode them, then let tidy re-encode them as XML-friendly _numerical_ entities; Dunno if there's a tidy option to re-convert already encoded entities.
  $data = html_entity_decode ( $data ) ;

  // Tidy
  if ( extension_loaded  ( "tidy" )) {
    // Specify configuration
    // The DOCTYPE syntax seems to cause complaints 
    // <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
    // is apparently illegal ? Need to omit it. Wants space inside the end of the tag ...
    // The xml declaration also causes trouble with browser rendering.
    // 
    $config = array(
               'indent'           => true,
               'output-xml'       => true,
               'numeric-entities' => true,
               'add-xml-decl'     => false,
               'doctype'          => 'omit',
               'char-encoding'    => 'utf8',
               'wrap'             => 200);


    $tidy = new tidy;
    $tidy->parseString($data, $config);
    $tidy->cleanRepair();
    return $tidy;
  } 
 
 // No Tidy Module, go commando
  if(defined("TIDY_PATH")){
    debug("No PHP tidy support, attempting to use the command-line way to process a string. This is pretty hokey!",2);
    // Need to write, pipe & read the result.
    $temp_path =  foreslash( $_ENV['TEMP']."/".tempnam("","htm") );
    file_put_contents($temp_path,$data);
    $out = xml_tidy_file($temp_path);
    unlink($temp_path);
    return $out;
  }

  debug("No html tidy support at all. You need to enable the PHP module to use this, or ensure the 'tidy' executable is present and defined as TIDY_PATH and use that",2);
  return $data;
}


/**
 * Tidy an xml file in place
 *
 * If we don't have the tidy module, we can still try to get the effect
 * using the command-line version.
 * Remote files are supported, they are fetched into a the temp dir
 * before being processed.
 *
 * @parameter $filepath string The full filepath of the file to be tidied
 * @returns string the content of the tidied file
 */
function xml_tidy_file($filepath)
{
  debug("Tidying file '$filepath' ",2);
  if(! is_local($filepath)){
    // OK, so it's a remote file and I have to process it on the command-line ...
    // Copy it down
    $source = file_get_contents($filepath);
    if(! $source){
      trigger_error("No content from '$filepath'", E_USER_WARNING ); return false;
    }
    debug("Retrieved file:$filepath is ".strlen($source)." big",3);
    // put it in a temp place
    $target_path =  $_ENV['TEMP']."/".tempnam("","htm");

    file_put_contents( foreslash($target_path) , $source ); 
  } else {
    $target_path = $filepath;
  }
  
  // format it
  $command = system_path(ensure_trailing_slash(TIDY_PATH)) ."tidy ".TIDY_OPTIONS ." \"". $target_path . "\"";
  $command = escapeshellcmd($command); 
  debug("Running \n$command" ,2); // running it without -m will pipe the results straight back to me.
  $out = `$command`;
  
  if(!$out){
    trigger_error("HTMLTidy failed to parse the input at all! It's probably very problematic HTML", E_USER_WARNING);
    $out = file_get_contents($target_path );
  }
  
  $out = xml_tidy_scripts($out);

  debug("<h3>After command-line tidy</h3><textarea rows='40' cols='80'>$out</textarea>",3);
  
  if($target_path != $filepath);
  { // remove temp file
    unlink($target_path); 
  }
  
  return $out;
}

/**
 * Manually fix some legacy HTML quirks that Tidy can't.
 *
 * Even after passing through Tidy, some scripts need escaping
 * with comments and CDATA before the result is true XHTML
 * Do this here with lots of messy regexps.
 * Put comment/CDATA wrappers around any inline scripts - replacing any existing possible comment escaping
 * ... probably can't handle already cdata-escaped cases...
 * .. :( Please replace this madness with something that makes sense
 * Test cases must include : Jscript with && in, Scripts already half-hidden by comments, empty (src-only) scripts, and src-only scripts containing comments.
 * @parameter $out string the source that needs fixing
 * @returns string This 'fixed' source.
 */
function xml_tidy_scripts($out){
  // Tidy still needs a little help : 
  $out = preg_replace("|(<script[^>]*>)(\s*)(<!--)?\s*(<!\\[CDATA\\[)?\s*|",   "$1\n<!"."--<![CDATA[\n$2",$out);
  $out = preg_replace("|\s*(\\]\\]>)?\s?(-->)?(\s*)(</script>)|",   "$3 // ]"."]> -->\n$4",$out);
  // SRC-only scripts are a pain - they didn't need this extra care, in fact it may break things (by causing 'nested' comments)
  // Remove what I've just done if it was empty.
  $out = preg_replace("|<\\!--<\\!\\[CDATA\\[ // \\]\\]> -->|", "",$out);
  // Note - if some script and noscript 'blocks' contain just comments no text whitespace,
  // some xml parsers (php4) delete the comments, 
  // then collapse empty script tags to <script ... /> ... which then breaks the browsers.  
  $out = preg_replace("|<noscript><|", "<noscript>\n<",$out);
  
  /*
  // Tidy also nicely fixes some empty tags with a closing slash ... but this
  // is too complex for the PHP4 XML parser is it doesn't have a bit of whitespace??
  $out = preg_replace("|(.)/>|", "$1 />",$out);
  // -- red herring, not the problem
  */  
  
  return $out;
}

?>