<?php
/**
 * @file
 * Utility for formatting XML files using XSL. 
 *
 * Compliant with the different XML implimentations in PHP versions 4 & 5.
 * 
 * @package coders
 * @author Dan Morrison dan@coders.co.nz
 * @version Aug 2005
 */

/*
 * For debug routines, parameter passing and some output formatting
 */
if(defined("DEBUGLEVEL")){
  require_once("debug.inc");
}
// and may conditionally call file-routines and tidy-functions.inc
// but only when this library is actually used.
//  require_once("file-routines.inc");

/**
 * Load whichever XML support is available
 */
function init_xsl(){
  // Just loading this lib means we really want to have XML
  // OK, support both old & new PHP library ways of doing things ...
  if (!extension_loaded('xsl') && !extension_loaded('xslt')) {
    if (!@dl('xslt.so')&&!@dl('php_xslt.dll')&&!@dl('php_xsl.dll')) {
      trigger_error("No XSL library support at all on this server!" , E_USER_ERROR );
      return FALSE;
    }
  }
  return TRUE;
}


/**
 * Try to load an xml doc from file, 
 *
 * Running tidy over the source first to override most parsing 
 * problems before they happen.
 * @parameter string Source filepath or URL
 * @parameter $do_tidy bool Whether to pre-process with HTMLTidy before parsing - ONLY do this with suspect HTML inout, not other XML sources.
 * @returns xmldoc
 */
function parse_in_xml_file($xmlfile,$do_tidy=0) {
  require_once("file-routines.inc");

  debug("Reading XML source from '$xmlfile'.",2);
  init_xsl();

  if(! file_exists($xmlfile)){
    trigger_error("File '$xmlfile' not found, cannot load XML source.",E_USER_WARNING);
    return false;
  }

  // Almost no content out there can be trusted to be valid :-(
  // I guess I'll tidy it myself before I attempt to parse it.

  if($do_tidy){
    include_once('tidy-functions.inc');
    if (! extension_loaded  ( "tidy" ) ){
      // No tidy module available. 
      // Old-school tidy must be run directly on the file, so do this first
      $xmlsource = xml_tidy_file($xmlfile);
    } else {
      $xmlsource = xml_tidy_string( file_get_contents($xmlfile) );
    }
  } else {
    $xmlsource = file_get_contents($xmlfile);
  }

  if(! trim($xmlsource)) { trigger_error("Failed to read contents of '$xmlfile' ",E_USER_WARNING);return;}
  else { debug("Retrieved source of '$xmlfile' OK, it's ".strlen($xmlsource)." characters",3);}

  debug("<h3>After loading text</h3><textarea rows='40' cols='80'>".$xmlsource."</textarea>",4);


  // If we force the source into the strict namespace, the XSL transformations 
  // (also in that namespace) SHOULD not litter the output with unwanted xmlns tags. 
  // so lets try and ensure the document knows that it's the right ns
  // $namespaceURI = "http://www.w3.org/TR/REC-html40";
  $namespaceURI = "http://www.w3.org/1999/xhtml"; // All XSL output should be this also, ok?

  // support PHP4 or 5 with their different xmls
  if ( extension_loaded  ( "dom" )) {
    // PHP 5 //
    $xmldoc = new DOMDocument; // DOM replacement PHP5 version
    debug("Using newer DOM extension (PHP5) to process XML",2);
    if (! $xmldoc->loadXML($xmlsource) ) {
      return debug_xml_source($text);
    }
    if ( isset($xmldoc->documentElement)) {
      $d = $xmldoc->documentElement ; // DOM replacement PHP5 version
      $d->namespaceURI = $namespaceURI ;
    }
    debug_pre($xmldoc->saveXML(), 4);
    debug("XML source successfuly loaded as an XML doc.",1);
    return $xmldoc;

  } else if ( extension_loaded ( "domxml" )) {
    // PHP 4 //
    debug("Using older DOM_XML extension (PHP4) to process XML",2);
    if ( ! $xmldoc = domxml_open_mem($xmlsource) ) {
      trigger_error("Failed to parse in xml source. ",E_USER_WARNING);
      debug( debug_xml_source($text) );
      return false;
    }

    if (is_callable(array($xmldoc,"document_object"))) {
      $d = $xmldoc->document_object(); // DOM XML PHP4 version
      $d->add_namespace( $namespaceURI,"" );
    } else if (is_callable(array($xmldoc,"root"))){
      $d = $xmldoc->root(); // DOM XML Old deprecated PHP4 version
      // $d->add_namespace( $namespaceURI,"" ); // xmlns:="blah" 
      // $d->add_namespace( $namespaceURI ); // <xxyz:html xmlns:xxyz="blah">
      // $d->set_namespace( $namespaceURI,"xhtml" ); // Arg - how to set defualt namespace
      // debug_pre( $xmldoc->dump_mem() );  

    } else {debug("Odd, no root available to fix the namespace of. Which PHP/domxml version is this again?",3);}

    debug("XML source successfuly loaded as an XML doc.",1);
    debug("<h3>After loading as XML</h3><textarea rows='40' cols='80'>".$xmldoc->dump_mem()."</textarea>",3);
    return $xmldoc; 

  } else {
    trigger_error("No known DOM XML support on this server at all - can't really do anything! We need to enable php module dom or domxml to parse web pages nicely");
    return false;
  }
}

function debug_xml_source($text){
    trigger_error("Still failed to parse contents of file :-( Give up.",E_USER_WARNING);
    print("<textarea rows='40' cols='80' wrap='none'>$text</textarea>");
}


/**
 * Version-safe string->xmldoc parser.
 * Convert a String to a DOM, via PHP4 or 5.
 * @parameter xmlsource $string Source XML
 * @parameter $do_tidy bool ONLY run HTMLTidy on HTML input, not once you already have XML source,
 * @returns xmldoc
 */
function parse_in_xml_string($xmlsource, $do_tidy=0) {
  if ($do_tidy){
    include_once('tidy-functions.inc');
    $xmlsource = xml_tidy_string($xmlsource);
  }
  
  init_xsl();
  // support PHP4 or 5 with their different xmls
  if ( extension_loaded  ( "dom" )) {
    // PHP 5 //
    $xmldoc = new DOMDocument; // DOM replacement PHP5 version
    $xmldoc->loadXML($xmlsource);
    return $xmldoc;
  } else if ( extension_loaded ( "domxml" )) {
    // PHP 4 //
    $xmldoc = domxml_open_mem($xmlsource);
    return $xmldoc; 
  } else {
    trigger_error("No known DOM XML support on this server at all - can't really do anything! We need to enable php module dom or domxml to parse web pages nicely");
    return false;
  }
}


/**
 * Do the actual conversion between XML+XSL
 *
 * Input and output are full DOM objects in PHP5
 * We return the result STRING, as that's what 
 * the process gives us :-/ 
 * Need to parse it back in again for pipelining.
 *
 * Painfully, the entire library support changed between PHP4 & 5.
 * We need to see if it's the "xsl" or the "xslt" library that's in use!
 *
 * @param DOMDocument or string $xmldoc 
 * @param DOMDocument or string $xsldoc 
 * @param array $parameters To be passed into the xslt_process()
 * @returns string The result.
 */
function xmldoc_plus_xsldoc($xmldoc,$xsldoc,$parameters)
{
  // OK, support both old & new PHP library ways of doing things ...

  init_xsl();
  if(extension_loaded("xslt")) {
    debug("Doing XSL using PHP4 xslt module",3);
    $xsltproc = xslt_create();
    xslt_set_encoding($xsltproc, 'UTF-8');
    xslt_set_base ( $xsltproc, "file://". $parameters['xsl_path']);
    debug("Set the XSLT processing base to "."file://". $parameters['xsl_path'],3);
    // xslt_process can take text or sockets, but it's not accepting DomDocuments today.
    // If We are handed DOMs, flatten it out and stream it in. (Isn't that a waste?)
    $args = array();
    $args['/_xml'] = (is_callable(array($xmldoc,"dumpmem")))?$xmldoc->dump_mem():$xmldoc;
    $args['/_xsl'] = (is_callable(array($xsldoc,"dumpmem")))?$xsldoc->dump_mem():$xsldoc;
    debug("Calling xslt_process() with parameters:".print_r($parameters,1),3);

    /*
    debug("<h2>INPUT</h2><textarea rows='20' cols='80'>{$args['/_xml']}</textarea>");
    debug("<h2>TRANSFORM</h2><textarea rows='20' cols='80'>{$args['/_xsl']}</textarea>");
    debug_pre($args);
    debug_pre($parameters);
    */
    
    $out = @xslt_process($xsltproc, 'arg:/_xml', 'arg:/_xsl', NULL, $args, $parameters);
    if (empty($out)) {
       die('XSLT processing error: '. xslt_error($xsltproc));
    }
    xslt_free($xsltproc);
    return $out;
  }

  if(extension_loaded("xsl")) {
    debug("Doing XSL using PHP5 xsl module",3);
    $xsltproc = new XSLTProcessor;
    xslt_set_base ( $xsltproc, $parameters['xsl_path']);
    
    $xsltproc->importStyleSheet($xsldoc); // attach the xsl rules
    // Set parameters when defined
    if ($parameters) {
       foreach ($parameters as $param => $value) {
         $xsltproc->setParameter("", $param, $value);
       }
    }
    // return $xsltproc->transformToXML($xml);
    return $xsltproc->transformToDOC($xmldoc);    
  }
}


/**
 * A version safe way to make an XML doc from a filename
 */
function load_xml_from_file($xmlfile){
  $xmldoc = new DomDocument('1.0');
  if(is_callable (array($xmldoc,"load"))){ // PHP5
    $xmldoc->load($xmlfile);
  } else {                                 // PHP4
    $xmldoc = domxml_open_file($xmlfile);
  }
  return $xmldoc;
}


?>